--- asterisk/channels/chan_mgcp.c.ORIG	2005-10-07 16:11:50.000000000 -0500
+++ asterisk/channels/chan_mgcp.c	2005-10-25 16:39:11.000000000 -0500
@@ -109,6 +109,7 @@
 #include "asterisk/utils.h"
 #include "asterisk/causes.h"
 #include "asterisk/dsp.h"
+#include "asterisk/devicestate.h"
 
 #ifndef IPTOS_MINCOST
 #define IPTOS_MINCOST 0x02
@@ -492,6 +493,7 @@
 static int mgcp_indicate(struct ast_channel *ast, int ind);
 static int mgcp_fixup(struct ast_channel *oldchan, struct ast_channel *newchan);
 static int mgcp_senddigit(struct ast_channel *ast, char digit);
+static int mgcp_devicestate(void *data);
 
 static const struct ast_channel_tech mgcp_tech = {
 	.type = type,
@@ -499,6 +501,7 @@
 	.capabilities = AST_FORMAT_ULAW,
 	.properties = AST_CHAN_TP_WANTSJITTER,
 	.requester = mgcp_request,
+	.devicestate = mgcp_devicestate,
 	.call = mgcp_call,
 	.hangup = mgcp_hangup,
 	.answer = mgcp_answer,
@@ -879,6 +882,54 @@
 	return res;
 }
 
+static int mgcp_devicestate(void *data)
+{
+	struct mgcp_gateway  *g;
+	struct mgcp_endpoint *e = NULL;
+	char *tmp, *endpt, *gw;
+	int ret = AST_DEVICE_INVALID;
+
+	endpt = ast_strdupa(data);
+	if ((tmp = strchr(endpt, '@'))) {
+		*tmp++ = '\0';
+		gw = tmp;
+	} else
+		goto error;
+
+	ast_mutex_lock(&gatelock);
+	g = gateways;
+	while (g) {
+		if (strcasecmp(g->name, gw) == 0) {
+			e = g->endpoints;
+			break;
+		}
+		g = g->next;
+	}
+
+	if (!e)
+		goto error;
+
+	while (e) {
+		if (strcasecmp(e->name, endpt) == 0)
+			break;
+		e = e->next;
+	}
+
+	if (!e)
+		goto error;
+
+	/*
+	 * As long as the gateway/endpoint is valid, we'll
+	 * assume that the device is available and its state
+	 * can be tracked.
+	 */
+	ret = AST_DEVICE_UNKNOWN;
+
+error:
+	ast_mutex_unlock(&gatelock);
+	return ret;
+}
+
 static int mgcp_call(struct ast_channel *ast, char *dest, int timeout)
 {
 	int res;
